#  Copyright: Copyright (c) 2020., Adam Jakab
#
#  Author: Adam Jakab <adam at jakab dot pro>
#  Created: 3/27/20, 3:52 PM
#  License: See LICENSE.txt

__author__ = u'Adam Jakab'
__email__ = u'adam@jakab.pro'
__copyright__ = u'Copyright (c) 2020, {} <{}>'.format(__author__, __email__)
__license__ = u'License :: OSI Approved :: MIT License'
__version__ = u'0.1.3'
__status__ = u'Kickstarted'

__PACKAGE_TITLE__ = u'Autofix'
__PACKAGE_NAME__ = u'beets-autofix'
__PACKAGE_DESCRIPTION__ = u'A beets plugin to execute repetitive tasks in one go.',
__PACKAGE_URL__ = u'https://github.com/adamjakab/BeetsPluginAutofix'
__PLUGIN_NAME__ = u'autofix'
__PLUGIN_ALIAS__ = None
__PLUGIN_SHORT_DESCRIPTION__ = u'execute repetitive tasks in one go'
